#If 0
== SHA256 ==
Secure 256-Bit hashing For PowerBASIC

Code For the following two files appears below:
SHA256.INC  Hash routines For returning 32-Byte SHA256 hashes
            Of buffers And files
SHA256.BAS  Test bed Exe illustrating buffer And file hashing
All Code compiles With either PBDLL 6.0 Or PBCC 2.0.

Available At the following URL Is a PDF file containing the NIST
specifications For the 256-Bit, 384-Bit, And 512-Bit extensions To
the SHA standard:
 http://csrc.nist.gov/cryptval/shs.html

A hash Is considered secure when it possesses the following qualities.
-- Determining the Input String From the hash (i.e., working backward
From the hash alone To determine the String which generated it) Is Not
considered feasible.
-- Given an Input String, it Is Not considered feasible To Find another
String which hashes To the same value.
-- It Is Not considered feasible To Find two Random strings which hash
To the same value.

An Input String can be Of Any length And thus far longer than its
resulting hash.  This Is accomplished through use Of a compression
Function which treats the Input As a combination Of Any previously
hashed Input And the current input.  Designing this feature Is one Of
the most important challenges when creating a secure hash algorithm.

Secure hashes have many uses, among them validation Of passphrases.
In this arrangement, a User enters a passphrase when setting up an
account.  The Input String Is hashed, And the hash Is stored (Not the
passphrase).  When the User seeks readmission, the passphrase entered
Is hashed, And the hash Is compared With the one On record.  If the
hashes match, the strings which generated them also must match.  If
the collection Of stored hashes Is compromised, the passphrases
remain unknown.

Secure hashes are Not designed For speed.  The implementation below
relies On assembly language To improve Speed In its most heavily
traveled section Of Code, but unless security Is required, a secure
hash Is a poor choice when compared With the many simpler, far more
efficient hash algorithms In widespread use.

Greg Turgeon
gturgeon@compuserve.com
1/10/2001
#EndIf

'=====================================================================
'-- SHA256.INC
'-- WIN32 API not required
'-- Uses no global data
'=====================================================================

Type tSHA_STATE
   state0   As Dword
   state1   As Dword
   state2   As Dword
   state3   As Dword
   state4   As Dword
   state5   As Dword
   state6   As Dword
   state7   As Dword
   pbuf     As Byte Ptr
   buflen   As Long
   pstate   As Dword Ptr
   k_array  As Long Ptr
End Type
%PBUF_OFFSET      = 32

%BLOCKSIZE        = 64     'bytes
%FILE_BUFFERSIZE  = 32000  'bytes

Declare Sub SHA_Buffer(ByVal pBuffer As Byte Ptr, ByVal Length&, ByVal Hash As Dword Ptr)
Declare Function SHA_File&(File_Name$, Hash$)
Declare Sub SHA_Init(ByVal pSHA_STATE As tSHA_STATE Ptr)
Declare Sub SHA_Compress(ByVal pSHA_State As tSHA_STATE Ptr)
Declare Function MakePadding$(ByVal TotalBytes&)


'====================
Sub SHA_Init(ByVal pSHA_STATE As tSHA_STATE Ptr)
Local p&

@pSHA_STATE.k_array = CodePtr(K_Array_Data)
p& = CodePtr(Init_Values)
@pSHA_STATE.pstate = pSHA_STATE
!  push  esi
!  push  edi
!  mov   esi, p&
!  mov   edi, pSHA_STATE
!  mov   ecx, 8
!  cld
!  rep   movsd
!  pop   edi
!  pop   esi
Exit Sub
'============
Init_Values:
! DD  &h6A09E667???, &hBB67AE85???, &h3C6EF372???, &hA54FF53A???
! DD  &h510E527F???, &h9B05688C???, &h1F83D9AB???, &h5BE0CD19???
K_Array_Data:
! DD  &h428a2f98???, &h71374491???, &hb5c0fbcf???, &he9b5dba5???, &h3956c25b???, &h59f111f1???
! DD  &h923f82a4???, &hab1c5ed5???, &hd807aa98???, &h12835b01???, &h243185be???, &h550c7dc3???
! DD  &h72be5d74???, &h80deb1fe???, &h9bdc06a7???, &hc19bf174???, &he49b69c1???, &hefbe4786???
! DD  &h0fc19dc6???, &h240ca1cc???, &h2de92c6f???, &h4a7484aa???, &h5cb0a9dc???, &h76f988da???
! DD  &h983e5152???, &ha831c66d???, &hb00327c8???, &hbf597fc7???, &hc6e00bf3???, &hd5a79147???
! DD  &h06ca6351???, &h14292967???, &h27b70a85???, &h2e1b2138???, &h4d2c6dfc???, &h53380d13???
! DD  &h650a7354???, &h766a0abb???, &h81c2c92e???, &h92722c85???, &ha2bfe8a1???, &ha81a664b???
! DD  &hc24b8b70???, &hc76c51a3???, &hd192e819???, &hd6990624???, &hf40e3585???, &h106aa070???
! DD  &h19a4c116???, &h1e376c08???, &h2748774c???, &h34b0bcb5???, &h391c0cb3???, &h4ed8aa4a???
! DD  &h5b9cca4f???, &h682e6ff3???, &h748f82ee???, &h78a5636f???, &h84c87814???, &h8cc70208???
! DD  &h90befffa???, &ha4506ceb???, &hbef9a3f7???, &hc67178f2???
End Sub


'====================
Sub SHA_Compress(ByVal pST As tSHA_STATE Ptr)
Static s_array&(), w_array&()
Static s As Long Ptr, w As Long Ptr
Local pk_array&, t0&, t1&, wi&, ki&, i&

If w& = 0 Then
   ReDim w_array&(63) : w = VarPtr(w_array&(0))
   ReDim s_array&(7)  : s = VarPtr(s_array&(0))
End If

'-- Copy for inline assembler
pk_array& = @pST.k_array

!  push  esi
!  push  edi
!  push  ebx

'-- Copy current state into s&()
!  mov   esi, pST           ;esi -> pST.state0
!  mov   edi, s
!  push  esi
!  mov   ecx, 8
!  cld
!  rep   movsd

'-- Copy current data block to w&() w/little-to-big endian conversion
!  pop   esi               ;esi -> st.state0
!  add   esi, %PBUF_OFFSET ;esi -> st.buf
!  mov   edi, w
!  mov   eax, [esi]
!  mov   esi, eax
!  mov   ecx, 16*4
SwapCopyTop:
!  sub   ecx, 4
!  mov   eax, [esi+ecx]
!  bswap eax
!  mov   [edi+ecx], eax
!  test  ecx, ecx
!  jnz   SwapCopyTop

'-- Fill W[16to63]
! mov   esi, w
! push  ebp
! mov   ebp, 16
FillTop:
!  mov   ebx, ebp
!  sub   ebx, 2
!  mov   eax, [esi]+[ebx*4]
!  mov   ecx, eax
!  mov   edx, ecx
!  sub   ebx, 13           ;prep for next access: (-2)+(-13) = -15
!  ror   eax, 17
!  ror   ecx, 19
!  shr   edx, 10
!  xor   eax, ecx
!  xor   eax, edx
!  mov   edi, eax          ;edi = temp total
!  mov   eax, [esi]+[ebx*4]
!  mov   ecx, eax
!  mov   edx, ecx
!  ror   eax, 7
!  ror   ecx, 18
!  shr   edx, 3
!  add   ebx, 8            ;= (-15)+8 = -7
!  xor   eax, ecx
!  xor   eax, edx
!  add   edi, eax

!  mov   eax, [esi]+[ebx*4]
!  sub   ebx, 9            ;= (-7)+(-9) = -16
!  add   edi, eax

!  mov   eax, [esi]+[ebx*4]
!  mov   ebx, ebp
!  add   eax, edi

!  inc   ebp
!  mov   [esi]+[ebx*4], eax
!  cmp   ebp, 63
!  jg    FillDone
!  jmp   FillTop
FillDone:
!  pop   ebp

'-- Compress: i& = 0 to 63
!  xor   eax, eax
!  mov   esi, s            ;here to CompressDone & END SUB: esi -> s[0]
!  mov   i&, eax
CompressTop:
!  mov   ebx, eax
!  mov   edx, w
!  mov   eax, [edx]+[ebx*4]
!  mov   wi&, eax
!  mov   edx, pk_array&
!  mov   eax, [edx]+[ebx*4]
!  mov   ki&, eax

!  mov   eax, [esi+16]
!  mov   ecx, [esi+20]
!  mov   ebx, eax
!  mov   edx, eax
!  and   eax, [esi+20]

!  not   edx
!  and   edx, [esi+24]
!  xor   eax, edx

!  mov   ecx, ebx
!  mov   ebx, eax
!  mov   eax, ecx

!  mov   edx, ecx
!  ror   eax, 6
!  ror   ecx, 11
!  ror   edx, 25

!  add   ebx, ki&
!  xor   eax, ecx
!  add   ebx, wi&
!  xor   eax, edx

!  add   eax, ebx
!  add   eax, [esi+28]

!  mov   t0&, eax

!  mov   eax, [esi]
!  mov   ecx, [esi+4]
!  mov   ebx, eax

!  mov   edx, eax
!  and   eax, ecx

!  and   edx, [esi+8]
!  xor   eax, edx

!  and   ecx, [esi+8]
!  xor   eax, ecx
!  mov   edx, eax

!  mov   eax, ebx
!  mov   ecx, ebx
!  ror   eax, 2
!  ror   ecx, 13
!  ror   ebx, 22
!  xor   eax, ecx
!  xor   eax, ebx

!  add   eax, edx
!  mov   t1&, eax

'  @s[7] = @s[6] : @s[6] = @s[5] : @s[5] = @s[4] : @s[4] = @s[3] + t0&
'  @s[3] = @s[2] : @s[2] = @s[1] : @s[1] = @s[0] : @s[0] = t0& + t1&
!  mov   edi, esi
!  mov   ecx, t0&

!  mov   eax, [esi+24]     ;@s[7] = @s[6]
!  mov   ebx, [esi+20]     ;@s[6] = @s[5]
!  mov   edx, [esi+16]     ;@s[5] = @s[4]
!  mov   [edi+28], eax
!  mov   [edi+24], ebx
!  mov   [edi+20], edx

!  mov   eax, [esi+12]     ;@s[4] = @s[3] + T0&
!  mov   ebx, [esi+8]      ;@s[3] = @s[2]
!  add   eax, ecx
!  mov   edx, [esi]        ;@s[1] = @s[0]
!  mov   [edi+12], ebx
!  mov   [edi+16], eax
!  mov   ebx, ecx          ;@s[0] = T0& + T1&
!  mov   eax, [esi+4]      ;@s[2] = @s[1]
!  mov   [edi+4], edx
!  add   ebx, t1&
!  mov   [edi+8], eax
!  mov   [edi], ebx

!  mov   eax, i&
!  inc   eax
!  cmp   eax, 63
!  jg    CompressDone
!  mov   i&, eax
!  jmp   CompressTop
CompressDone:

'-- Add current state s() to context
'  for i& = 0 to 7 : @pST.@pstate[i&] = @pST.@pstate[i&] + @s[i&] : next i&
!  mov   edi, pST
!  mov   ecx, 8*4
AddCopyTop:
!  sub   ecx, 4
!  mov   eax, [edi+ecx]
!  add   eax, [esi+ecx]
!  mov   [edi+ecx], eax
!  test  ecx, ecx
!  jnz   AddCopyTop

!  pop   ebx
!  pop   edi
!  pop   esi
End Sub


'====================
Sub SHA_Buffer(ByVal DataBuffer As Byte Ptr, ByVal Length&,ByVal Hash As Dword Ptr) Export
'-- Expects parameter Hash to point to buffer of correct size of 32 bytes (256bits \ 8)
Register i&
Local lastbuff$
Local st As tSHA_STATE, p&

i& = Length& And (%BLOCKSIZE-1)
lastbuff$ = Peek$((DataBuffer + Length&) - i&, i&)
lastbuff$ = lastbuff$ + MakePadding$(Length&)

SHA_Init ByVal VarPtr(st)

st.buflen = Length&
st.pbuf = DataBuffer

i& = Length& And (Not %BLOCKSIZE-1)
Do While i&
   SHA_Compress ByVal VarPtr(st)
   st.pbuf = st.pbuf + %BLOCKSIZE
   i& = i& - %BLOCKSIZE
Loop

st.buflen = Len(lastbuff$)
st.pbuf = StrPtr(lastbuff$)

Do While st.buflen
   SHA_Compress ByVal VarPtr(st)
   st.pbuf = st.pbuf + %BLOCKSIZE
   st.buflen = st.buflen - %BLOCKSIZE
Loop

'-- Copy current state (as dwords) from s&() to Hash
'   for i& = 0 to 7 : @Hash[i&] = st.@pstate[i&] : next i&
p& = st.pstate
!  push  esi
!  push  edi
!  mov   esi, p&           ;esi -> st.state0
!  mov   edi, Hash
!  mov   ecx, 8
!  cld
!  rep   movsd
!  pop   edi
!  pop   esi
End Sub


'====================
Function SHA_File&(File_Name$, Hash$) Export
'-- Returns 0 on success, or PB (not OS) error code
Register i&, bytesleft&
Local buffer$, padding$
Local st As tSHA_STATE, phash As Dword Ptr
Local infile&, ecode&, lastpass&, maxstring&

'-- If file not found, return PB error code
If Len(Dir$(File_Name$)) = 0 Then
   Function = 53 : Exit Function
End If

buffer$ = String$(%FILE_BUFFERSIZE, 0)
maxstring& = %FILE_BUFFERSIZE

st.buflen = %BLOCKSIZE
SHA_Init ByVal VarPtr(st)

infile& = FreeFile
Open File_Name$ For Binary Lock Shared As infile& Base=0
If Err Then GoTo SHA_File_Error
bytesleft& = Lof(infile&)
padding$ = MakePadding$(bytesleft&)

Do
   'Resize if necessary & flag final buffer
   If bytesleft& =< maxstring& Then
      maxstring& = bytesleft&
      buffer$ = String$(maxstring&, 0)
      Incr lastpass&
   End If
   Get infile&,, buffer$ : If Err Then GoTo SHA_File_Error
   If lastpass& Then buffer$ = buffer$ + padding$
   st.pbuf = StrPtr(buffer$)
   For i& = 1 To (Len(buffer$) \ %BLOCKSIZE)
      SHA_Compress ByVal VarPtr(st)
      st.pbuf = st.pbuf + %BLOCKSIZE
   Next i&
   bytesleft& = bytesleft& - maxstring&
Loop Until lastpass&
Close infile& : If Err Then GoTo SHA_File_Error

'-- Copy current state (as dwords) from s&() to Hash$
'for i& = 0 to 7 : @Hash[i&] = st.@pstate[i&] : next i&
Hash$ = String$(32,0)
phash = StrPtr(Hash$)
lastpass& = st.pstate
!  push  esi
!  push  edi
!  mov   esi, lastpass&           ;esi -> st.state0
!  mov   edi, phash
!  mov   ecx, 8
!  cld
!  rep   movsd
!  pop   edi
!  pop   esi

Exit_SHA_File:
Function = ecode&
Exit Function

'============
SHA_File_Error:
If Err Then
   ecode& = ErrClear
Else
   ecode& = -1
End If
Resume Exit_SHA_File
End Function


'=========================
Function MakePadding$(ByVal TotalBytes&)
'-- Creates the necessary string to append to buffer being hashed
Local buffbits&&, padding$
Local pbyte1 As Byte Ptr, pbyte2 As Byte Ptr, padbytes&, i&

buffbits&& = TotalBytes& * 8
padding$ = String$(8,0)
pbyte1 = StrPtr(padding$) : pbyte2 = VarPtr(buffbits&&)

'-- Copy bytes in reverse
For i& = 0 To 7
   @pbyte1[i&] = @pbyte2[7 - i&]
Next i&

padbytes& = %BLOCKSIZE - ((TotalBytes&+9) And (%BLOCKSIZE-1))
Function = Chr$(&h80) + String$(padbytes&,0) + padding$
End Function
'-- END SHA256.INC ---------------------------------------------------
