'Adler-32 is composed of two sums accumulated per byte: s1 is the sum
'of all bytes, s2 is the sum of all s1 values. Both sums are done
'modulo 65521. s1 is initialised to 1, s2 to zero. The Adler-32 checksum
'is stored as s2*65536 + s1 in most-significant-byte first (network) order.
'
'The algorithm is much faster than the CRC32 algorithm yet still provides
'an extremely low probability of undetected errors.
'
'The modulo on unsigned long accumulators can be delayed for 5552 bytes,
'so the modulo operation time is negligible. If the bytes are a, b, c,
'the second sum is 3a + 2b + c + 3, and so is position and order sensitive,
'unlike the first sum, which is just a checksum. That 65521 is prime is
'important to avoid a possible large class of two-byte errors that leave
'the check unchanged. (The Fletcher checksum uses 255, which is not prime
'and which also makes the Fletcher check insensitive to single byte
'changes 0-255).
'
'The sum s1 is initialised to 1 instead of zero to make the length of the
'sequence part of s2, so that the length does not have to be checked
'separately. (Any sequence of zeroes has a Fletcher checksum of zero.)")
'

#Compile Exe  'PBCC.  Replace STDOUT with MSGBOX to compile with PBDLL
#Include "win32api.inc"

Function Adler32(sBuffer As String) As Long
    Dim Seed As Long
    Dim S1 As Long
    Dim S2 As Long
    Dim N As Long
    %CrcBase = 65521
    Seed = 1
    S1 = Seed And &HFFFF
    S2 = (Seed / 65536) And &HFFFF
    For N = 1 To Len(sBuffer)
        S1 = (S1 + Asc(Mid$(sBuffer, N,1))) Mod %CrcBase
        S2 = (S2 + S1) Mod %CrcBase
    Next
    Adler32 = (S2 * 65536) + S1
End Function

Function Adler32asm(sBuffer As String) As Long
#Register None
Dim Seed As Long, S1 As Long, S2 As Long, N As Long
Dim sPtr As Long
sPtr = StrPtr(sBuffer) - 1
N = Len(sBuffer) + sPtr
  '// S1 = Seed AND &HFFFF
    ! mov   Seed, 1
    ! xor   eax, eax
    ! add   eax, Seed
    ! mov   ebx, eax
    ! and   eax, &hFFFF
    ! mov   eax, ebx
    ! mov   S1, eax
  '// S2 = (Seed / 65536) AND &HFFFF
    ! mov   eax, Seed
    ! mov   ebx, &h10000
    ! xor   edx, edx
    ! div   ebx
    ! and   eax, &hFFFF
    ! mov   S2, eax
    ! mov   ecx, sPtr
    StartLoop: '// FOR N = 1 TO LEN(sBuffer)
    ! inc   ecx
    ! cmp   ecx, N
    ! jg    EndLoop
      '//S1 = (S1 + ASC(MID$(sBuffer, N,1))) MOD %CrcBase
        ! mov   eax, S1
        ! xor   ebx, ebx
        ! mov   bl, [ecx]
        ! add   eax, ebx
        ! mov   ebx, &hFFF1
        ! xor   edx, edx
        ! div   ebx
        ! mov   S1, edx
      '//S2 = (S2 + S1) MOD %CrcBase
        ! mov   eax, S2
        ! add   eax, S1
        ! mov   ebx, &hFFF1
        ! xor   edx, edx
        ! div   ebx
        ! mov   S2, edx
        ! jmp   StartLoop   ;// NEXT
    EndLoop:
    '// FUNCTION = (S2 * 65536) + S1
        ! mov   eax, S2
        ! mov   ecx, &h10000
        ! mul   ecx
        ! add   eax, S1
        ! mov   N, eax
    Function = N
End Function

Function PBMain() As Long
On Error Resume Next
Dim TestBuffer As String
Dim Adler As Long
TestBuffer = Chr$(17) & Chr$(34) & Chr$(51)
Adler = Adler32(TestBuffer)    'PB basic code
'Adler = Adler32asm(TestBuffer) 'PB inline asm
If Adler = 11337831 Then
    STDOUT "Calculated correctly"
Else
    STDOUT "Calculation failed"
End If
STDOUT "Adler32 checksum=" & Str$(Adler)
End Function
