#If 0
=====================================================================
                        Rijndael Encryption
=====================================================================
Rijndael has been selected by the NIST As "the AES [Advanced
Encryption Standard] algorithm" and is described in FIPS-197, which is
available here: http://csrc.nist.gov/publications/fips/fips197/fips-197.pdf

Rijndael Is Free For Any use public Or Private, commercial Or
non-commercial.  The authors Of Rijndael provide a home Page For the
algorithm: http://www.esat.kuleuven.ac.be/~rijmen/rijndael/

The PowerBASIC implementation appearing below, which meets the AES
standard For 16-Byte block operations, Is based In part On java
routines written And placed In the public domain by Paulo Barreto.
The PB implementation Is hereby placed In the public domain.  Use it
As you wish.  My hope Is discourage reliance On home-grown encryption
schemes In favor Of well-examined, strong, freely available
algorithms.

In this posting, test-bed Code appears below the RIJNDAEL.BAS file
contents.  All Code requires compiler releases 3.0/7.0 Or later.

 Implementation Notes
-- The algorithm operates On plaintext blocks Of 16 bytes.  Encryption
Of shorter blocks Is possible Only by padding the plaintext (often
With zero bytes), which can be accomplished through several methods.
The simplest Of them assumes that the final Byte Of plaintext always
identifies the number Of bytes Of padding added, including the final
Byte itself.

Examples:
   total plaintext bytes         = 30
   plaintext blocks encrypted:   = 2
   final block                   = Chr$(x1 To x14) + Chr$(0,2)
   total plaintext bytes         = 40
   plaintext blocks encrypted:   = 3
   final block                   = Chr$(x1 To x8) + Chr$(0,0,0,0,0,0,0,8)
-- Encryption key lengths can Range From 1 To 32 bytes (8 To 256 Bits).
-- Implementation here Is handled through an #Include file.  No Global
Data is employed.
-- As presented, the Code does Not supply a ready-To-use encryption
application.  It offers necessary pieces Only, As well As an
illustration Of their use.  Always keep In mind that most encryption
Is broken because Of implementation flaws And weaknesses.
-- Like most encryption algorithms, Rijndael was designed On
big-endian systems.  For this reason, little-endian systems Return
correct test vector results Only through considerable Byte-swapping,
With efficiency suffering As a result.  Because it adds Nothing To the
encryption security, an efficient implementation should avoid the
Byte-swapping.  Here the equate %VERIFY_IMPLEMENTATION can be remmed
To avoid this Byte-swapping.
-- Because Of the methods Of Data handling required for most
encryption And hashing, PowerBASIC's LONGs should be used to assure
correct Bit-level results (As well As additional Speed).
Greg Turgeon 08/2002
#EndIf
'-- Rem this equate for any use other than verification of implementation.
'#########################################################################



%VERIFY_IMPLEMENTATION = 1
DefLng A-Z
%MAX_ROUNDS    = 14
%MIN_ROUNDS    = 10
%BLOCKSIZE     = 16
%SUBKEY_SIZE   = (%BLOCKSIZE*2*8)   '256
%MAX_KEY_BYTES = (%BLOCKSIZE*2)
%ENCRYPT       = 1
%DECRYPT       = 2

Type ENCRYPTION_CONTEXT
   Action        As Long
   Rounds        As Long
   UserKeyLength As Long
   UserKey       As Long Ptr
   InBlock       As Long Ptr
   OutBlock      As Long Ptr
   KeyBlocks     As Long                  '4*4-byte blocks
   KE            As Long Ptr              '<-- pointer to encryption subkey buffer
   KE_Buffer     As String * %SUBKEY_SIZE '<-- encryption subkey buffer
   KD            As Long Ptr              '<-- pointer to decryption subkey buffer
   KD_Buffer     As String * %SUBKEY_SIZE '<-- decryption subkey buffer
   Se            As Byte Ptr
   Sd            As Byte Ptr
   Te0           As Long Ptr
   Te1           As Long Ptr
   Te2           As Long Ptr
   Te3           As Long Ptr
   Td0           As Long Ptr
   Td1           As Long Ptr
   Td2           As Long Ptr
   Td3           As Long Ptr
   Rcon          As Long Ptr
End Type

'--------------------
'-- Utility macros
'--------------------
Macro zbs(xx)=String$(xx,0)
'--------------------

Macro Function shiftlc(xx,constant_shiftval)
retval = (xx)
!  shl   retval, constant_shiftval
End Macro = retval
'--------------------

Macro Function shiftrc(xx,constant_shiftval)
retval = (xx)
!  shr   retval, constant_shiftval
End Macro = retval
'--------------------

Macro Function byte3(xx)
retval = (xx)
!  mov   eax, retval
!  shr   eax, 24
!  mov   retval, eax
End Macro = retval
'--------------------

Macro Function byte2(xx)
retval = (xx)
!  mov   eax, retval
!  shr   eax, 16
!  and   eax, &hff
!  mov   retval, eax
End Macro = retval
'--------------------

Macro Function byte1(xx)
retval = (xx)
!  mov   eax, retval
!  shr   eax, 8
!  and   eax, &hff
!  mov   retval, eax
End Macro = retval
'--------------------

Macro Function byte0(xx)
retval = (xx)
!  mov   eax, retval
!  and   eax, &hff
!  mov   retval, eax
End Macro = retval
'--------------------

Macro Function get_block(mem_ptr)
!  mov   edx, mem_ptr
!  mov   eax, [edx]
#If %Def(%VERIFY_IMPLEMENTATION)
!  bswap eax
#EndIf
!  mov   retval, eax
!  add   edx, 4
!  mov   mem_ptr, edx
End Macro = retval
'--------------------

Macro put_block(mem_ptr,value)
retval=value
!  mov   edx, retval
#If %Def(%VERIFY_IMPLEMENTATION)
!  bswap edx
#EndIf
!  mov   eax, mem_ptr
!  mov   [eax], edx
!  add   eax, 4
!  mov   mem_ptr, eax
End Macro
Declare Function Rijndael_Init&(Ctx As ENCRYPTION_CONTEXT)
Declare Function Set_Key&(Ctx As ENCRYPTION_CONTEXT)
Declare Function MakeDecryptKeySchedule(Ctx As ENCRYPTION_CONTEXT)
Declare Function EncryptBlock&(Ctx As ENCRYPTION_CONTEXT)
Declare Function DecryptBlock&(Ctx As ENCRYPTION_CONTEXT)
'====================

Function Set_Key&(Ctx As ENCRYPTION_CONTEXT)
Local i&, n&, nk&, pblock&, r&, temp&, t&, u&, v&, retval&
Local in_ukey$, in_key As Long Ptr, k As Long Ptr, s As Byte Ptr
Select Case As Long Ctx.UserKeyLength
   Case 16
      Ctx.Rounds = 10
   Case 24
      Ctx.Rounds = 12
   Case 32
      Ctx.Rounds = 14
   Case Else
      Function = 0
      Exit Function
End Select
Ctx.KeyBlocks = (Ctx.Rounds+1)*4
Rijndael_Init Ctx
in_ukey$ = zbs(%MAX_KEY_BYTES)
in_key = StrPtr(in_ukey$)
pblock = Ctx.UserKey
k = Ctx.KE  'local copy
For i = 0 To (Ctx.Rounds-6)-1
   @k[i] = get_block(pblock)
Next i
s = Ctx.Se
i = Ctx.Rounds - 6 : nk = i : n = 0
Do While i < Ctx.KeyBlocks
   temp = @k[i-1]
   If n = 0 Then
      n = Nk
      t = byte2(temp)
      v = shiftlc(@s[t],24)
      t = byte1(temp) : u = shiftlc(@s[t],16)
      v = v Or u
      t = byte0(temp) : u = shiftlc(@s[t],8)
      v = v Or u
      u = @s[byte3(temp)]
      temp = (v Or u) Xor Ctx.@Rcon[r] : Incr r
   Else
      If ((Nk = 8) And (n = 4)) Then
         t = byte3(temp)
         v = shiftlc(@s[t],24)
         t = byte2(temp) : u = shiftlc(@s[t],16)
         v = v Or u
         t = byte1(temp) : u = shiftlc(@s[t],8)
         v = v Or u
         u = @s[temp And &hff]
         temp = v Or u
      End If
   End If
   @k[i] = @k[i-nk] Xor temp
   Incr i : Decr n
Loop
If Ctx.Action = %DECRYPT Then MakeDecryptKeySchedule Ctx
Function = -1
End Function
'--------------------

Macro Function imix(tt)
u =       Ctx.@Td0[Ctx.@Se[byte3(tt)]]
u = u Xor Ctx.@Td1[Ctx.@Se[byte2(tt)]]
u = u Xor Ctx.@Td2[Ctx.@Se[byte1(tt)]]
u = u Xor Ctx.@Td3[Ctx.@Se[tt And &hff]]
End Macro = u
'====================

Function MakeDecryptKeySchedule(Ctx As ENCRYPTION_CONTEXT)
Local d&, e&, i&, retval&, t&, u&, k As Long Ptr
k = Ctx.KD
d = 0 : e = Ctx.Rounds*4
@k[d]   = Ctx.@KE[e]   : @k[d+1] = Ctx.@KE[e+1]
@k[d+2] = Ctx.@KE[e+2] : @k[d+3] = Ctx.@KE[e+3]
d = d + 4 : e = e - 4
For i = 1 To Ctx.Rounds-1
   t = Ctx.@KE[e]   : @k[d]   = imix(t)
   t = Ctx.@KE[e+1] : @k[d+1] = imix(t)
   t = Ctx.@KE[e+2] : @k[d+2] = imix(t)
   t = Ctx.@KE[e+3] : @k[d+3] = imix(t)
   d = d + 4 : e = e - 4
Next i
@k[d]   = Ctx.@KE[e]   : @k[d+1] = Ctx.@KE[e+1]
@k[d+2] = Ctx.@KE[e+2] : @k[d+3] = Ctx.@KE[e+3]
End Function
'--------------------

Macro Function packbytes(aa,bb,cc,dd)
aa = shiftlc(aa,24)
bb = shiftlc(bb,16)
cc = shiftlc(cc,8)
End Macro = (aa Or bb Or cc Or dd)
'--------------------

Macro Function enc_ta(aa,bb,cc,dd)
u =       Ctx.@Te0[byte3(aa)]
u = u Xor Ctx.@Te1[byte2(bb)]
u = u Xor Ctx.@Te2[byte1(cc)]
u = u Xor Ctx.@Te3[dd And &hff]
End Macro = u
'--------------------

Macro Function enc_tal(aa,bb,cc,dd,ww)
u = Ctx.@Se[byte3(aa)] : v = shiftrc(ww,24)
a0 = u Xor v
u = Ctx.@Se[byte2(bb)] : v = shiftrc(ww,16)
a1 = (u Xor v) And &hff
u = Ctx.@Se[byte1(cc)] : v = shiftrc(ww,8)
a2 = (u Xor v) And &hff
u = Ctx.@Se[dd And &hff] : v = ww
a3 = (u Xor v) And &hff
retval = packbytes(a0,a1,a2,a3)
End Macro = retval
'====================

Function EncryptBlock&(Ctx As ENCRYPTION_CONTEXT)
Local r&, t&, u&, v&, w&, pblock&, retval&, k As Long Ptr
Local a0&, a1&, a2&, a3&, t0&, t1&, t2&, t3&, s0&, s1&, s2&, s3&
pblock = Ctx.InBlock
t0 = get_block(pblock) : t1 = get_block(pblock)
t2 = get_block(pblock) : t3 = get_block(pblock)
k = Ctx.KE
t0 = t0 Xor @k[0] : t1 = t1 Xor @k[1]
t2 = t2 Xor @k[2] : t3 = t3 Xor @k[3]
For r = 1 To Ctx.Rounds-1
   k = k + (4*4)
   t = enc_ta(t0,t1,t2,t3) : a0 = t Xor @k[0]
   t = enc_ta(t1,t2,t3,t0) : a1 = t Xor @k[1]
   t = enc_ta(t2,t3,t0,t1) : a2 = t Xor @k[2]
   t = enc_ta(t3,t0,t1,t2) : a3 = t Xor @k[3]
   t0 = a0 : t1 = a1 : t2 = a2 : t3 = a3
Next r
k = k + (4*4)
w = @k[0] : s0 = enc_tal(t0,t1,t2,t3,w)
w = @k[1] : s1 = enc_tal(t1,t2,t3,t0,w)
w = @k[2] : s2 = enc_tal(t2,t3,t0,t1,w)
w = @k[3] : s3 = enc_tal(t3,t0,t1,t2,w)
pblock = Ctx.OutBlock
put_block(pblock,s0) : put_block(pblock,s1)
put_block(pblock,s2) : put_block(pblock,s3)
End Function
'--------------------

Macro Function dec_ta(aa,bb,cc,dd)
u =       Ctx.@Td0[byte3(aa)]
u = u Xor Ctx.@Td1[byte2(bb)]
u = u Xor Ctx.@Td2[byte1(cc)]
u = u Xor Ctx.@Td3[dd And &hff]
End Macro = u
'--------------------

Macro Function dec_tal(aa,bb,cc,dd,ww)
u = Ctx.@Sd[byte3(aa)] : v = shiftrc(ww,24)
a0 = u Xor v
u = Ctx.@Sd[byte2(bb)] : v = shiftrc(ww,16)
a1 = (u Xor v) And &hff
u = Ctx.@Sd[byte1(cc)] : v = shiftrc(ww,8)
a2 = (u Xor v) And &hff
u = Ctx.@Sd[dd And &hff] : v = ww
a3 = (u Xor v) And &hff
retval = packbytes(a0,a1,a2,a3)
End Macro = retval
'====================

Function DecryptBlock&(Ctx As ENCRYPTION_CONTEXT)
Local r&, t&, u&, v&, w&, pblock&, retval&, k As Long Ptr
Local a0&, a1&, a2&, a3&, t0&, t1&, t2&, t3&, s0&, s1&, s2&, s3&
pblock = Ctx.InBlock
t0 = get_block(pblock) : t1 = get_block(pblock)
t2 = get_block(pblock) : t3 = get_block(pblock)
k = Ctx.KD
t0 = t0 Xor @k[0] : t1 = t1 Xor @k[1]
t2 = t2 Xor @k[2] : t3 = t3 Xor @k[3]
For  r = 1 To Ctx.Rounds-1
   k = k + (4*4)
   t = dec_ta(t0,t3,t2,t1) : a0 = t Xor @k[0]
   t = dec_ta(t1,t0,t3,t2) : a1 = t Xor @k[1]
   t = dec_ta(t2,t1,t0,t3) : a2 = t Xor @k[2]
   t = dec_ta(t3,t2,t1,t0) : a3 = t Xor @k[3]
   t0 = a0 : t1 = a1 : t2 = a2 : t3 = a3
Next r
k = k + (4*4)
w = @k[0] : s0 = dec_tal(t0,t3,t2,t1,w)
w = @k[1] : s1 = dec_tal(t1,t0,t3,t2,w)
w = @k[2] : s2 = dec_tal(t2,t1,t0,t3,w)
w = @k[3] : s3 = dec_tal(t3,t2,t1,t0,w)
pblock = Ctx.OutBlock
put_block(pblock,s0) : put_block(pblock,s1)
put_block(pblock,s2) : put_block(pblock,s3)
End Function
'====================

Function Rijndael_Init&(Ctx As ENCRYPTION_CONTEXT)
Static table_buffer$
Static se As Byte Ptr, te0 As Long Ptr, te1 As Long Ptr, te2 As Long Ptr,  te3 As Long Ptr
Static sd As Byte Ptr, td0 As Long Ptr, td1 As Long Ptr, td2 As Long Ptr,  td3 As Long Ptr
Static rcon As Long Ptr
Ctx.KE_Buffer = zbs(%SUBKEY_SIZE)
Ctx.KE        = VarPtr(Ctx.KE_Buffer)
Ctx.KD_Buffer = zbs(%SUBKEY_SIZE)
Ctx.KD        = VarPtr(Ctx.KD_Buffer)
If te0 = 0 Then GoSub MakeTables
Ctx.Se        = se
Ctx.Sd        = sd
Ctx.Te0       = te0
Ctx.Te1       = te1
Ctx.Te2       = te2
Ctx.Te3       = te3
Ctx.Td0       = td0
Ctx.Td1       = td1
Ctx.Td2       = td2
Ctx.Td3       = td3
Ctx.Rcon      = rcon
Exit Function
'============
MakeTables:
Local ss As Byte Ptr
Local root&, s1&, s2&, s3&, i1&, i2&, i4&, i8&, i9&, ib&, id&, ie&, t&, u&, v&, retval&
table_buffer$ = zbs((1024*8)+(256*2)+(40))
te0 = StrPtr(table_buffer)
te1 = (te0+1024)   : te2 = te0+(1024*2) : te3  = te0+(1024*3)
td0 = te0+(1024*4) : td1 = te0+(1024*5) : td2  = te0+(1024*6) : td3 = te0+(1024*7)
se  = te0+(1024*8) : sd  = se+256       : rcon = se+(256*2)
Root = &h11B
ss = CodePtr(SS_Table)
For i1 = 0 To 255
   s1 = @ss[i1]
   s2 = shiftlc(s1,1)
   If (s2 >= &h100) Then s2 = s2 Xor Root
   s3 = s2 Xor s1
   i2 = shiftlc(i1,1)
   If (i2 >= &h100) Then i2 = i2 Xor Root
   i4 = shiftlc(i2,1)
   If (i4 >= &h100) Then i4 = i4 Xor Root
   i8 = shiftlc(i4,1)
   If (i8 >= &h100) Then i8 = i8 Xor Root
   i9 = i8 Xor i1
   ib = i9 Xor i2
   Id = i9 Xor i4
   ie = i8 Xor i4 Xor i2
   @se[i1] = s1 And 255
   u = shiftlc(s2,24)
   v = shiftlc(s1,16)
   t = u Or v
   u = shiftlc(s1,8)
   t = t Or u Or s3
   @te0[i1] = t
   u = shiftrc(t,8)
   v = shiftlc(t,24)
   @te1[i1] = u Or v
   u = shiftrc(t,16)
   v = shiftlc(t,16)
   @te2[i1] = u Or v
   u = shiftrc(t,24)
   v = shiftlc(t,8)
   @te3[i1] = u Or v
   @sd[s1] = i1 And 255
   u = shiftlc(ie,24)
   v = shiftlc(i9,16)
   t = u Or v
   u = shiftlc(Id,8)
   t = t Or u Or ib
   @td0[s1] = t
   u = shiftrc(t,8)
   v = shiftlc(t,24)
   @td1[s1] = u Or v
   u = shiftrc(t,16)
   v = shiftlc(t,16)
   @td2[s1] = u Or v
   u = shiftrc(t,24)
   v = shiftlc(t,8)
   @td3[s1] = u Or v
Next i1
Local i&, r&
r = 1
@rcon[0] = shiftlc(r,24)
For i = 1 To 9
   r = shiftlc(r,1)
   If (r >= &h100) Then r = r Xor Root
   @rcon[i] = shiftlc(r,24)
Next i
Return
SS_Table:
! DD  &h7b777c63, &hc56f6bf2, &h2b670130, &h76abd7fe
! DD  &h7dc982ca, &hf04759fa, &hafa2d4ad, &hc072a49c
! DD  &h2693fdb7, &hccf73f36, &hf1e5a534, &h1531d871
! DD  &hc323c704, &h9a059618, &he2801207, &h75b227eb
! DD  &h1a2c8309, &ha05a6e1b, &hb3d63b52, &h842fe329
! DD  &hed00d153, &h5bb1fc20, &h39becb6a, &hcf584c4a
! DD  &hfbaaefd0, &h85334d43, &h7f02f945, &ha89f3c50
! DD  &h8f40a351, &hf5389d92, &h21dab6bc, &hd2f3ff10
! DD  &hec130ccd, &h1744975f, &h3d7ea7c4, &h73195d64
! DD  &hdc4f8160, &h88902a22, &h14b8ee46, &hdb0b5ede
! DD  &h0a3a32e0, &h5c240649, &h62acd3c2, &h79e49591
! DD  &h6d37c8e7, &ha94ed58d, &heaf4566c, &h08ae7a65
! DD  &h2e2578ba, &hc6b4a61c, &h1f74dde8, &h8a8bbd4b
! DD  &h66b53e70, &h0ef60348, &hb9573561, &h9e1dc186
! DD  &h1198f8e1, &h948ed969, &he9871e9b, &hdf2855ce
! DD  &h0d89a18c, &h6842e6bf, &h0f2d9941, &h16bb54b0
End Function
'-- end RIJNDAEL.INC
