
'=====================================================================
'                       CAST-128 Test Bed code
'               Compiles with either PBWIN 7.0+ or PBCC 3.0+
'=====================================================================
#Compile Exe
#Register None
#Dim All
'============
DefLng A-Z

'MACRO zbs(x)=string$(x,0)

#Include "CAST-128.INC"

Declare Function Hex2Show$(Buffer$)

'====================
Function PBMain&()
Register i&
Local key$, plain$, cipher$, shouldbe$, t$
Local ctx As ENCRYPTION_CONTEXT  ' defined in CAST-128.BAS
EnterCC
CAST128_Init ctx
'-- Standard CAST-128 test vectors:
'-- 128-bit key
key      = Chr$(&h01,&h23,&h45,&h67,&h12,&h34,&h56,&h78,&h23,&h45,&h67,&h89,&h34,&h56,&h78,&h9A)
plain    = Chr$(&h01,&h23,&h45,&h67,&h89,&hAB,&hCD,&hEF)
cipher   = zbs(Len(plain$))
shouldbe = Chr$(&h23,&h8B,&h4F,&hE5,&h84,&h7E,&h44,&hB2)
t = "Key length (bits):" + Str$(Len(key)*8) + eol
t = t + "plain:    " + Hex2Show$(plain$) + eol
GoSub DoEncrypt
t = t + "cipher:   " + Hex2Show$(cipher) + eol
t = t + "shouldbe: " + Hex2Show$(shouldbe) + eol
'-- Be sure to reload key$
key = Chr$(&h01,&h23,&h45,&h67,&h12,&h34,&h56,&h78,&h23,&h45,&h67,&h89,&h34,&h56,&h78,&h9A)
GoSub DoDecrypt
t = t + "plain:    " + Hex2Show$(plain) + eol + eol
'-- 80-bit key
key      = Chr$(&h01,&h23,&h45,&h67,&h12,&h34,&h56,&h78,&h23,&h45)
plain    = Chr$(&h01,&h23,&h45,&h67,&h89,&hAB,&hCD,&hEF)
cipher   = zbs(Len(plain))
shouldbe = Chr$(&hEB,&h6A,&h71,&h1A,&h2C,&h02,&h27,&h1B)
t = t + "Key length (bits):" + Str$((Len(key$)*8)) + eol
t = t + "plain:    " + Hex2Show$(plain) + eol
GoSub DoEncrypt
t = t + "cipher:   " + Hex2Show$(cipher) + eol
t = t + "shouldbe: " + Hex2Show$(shouldbe) + eol
key = Chr$(&h01,&h23,&h45,&h67,&h12,&h34,&h56,&h78,&h23,&h45)
GoSub DoDecrypt
t = t + "plain:    " + Hex2Show$(plain) + eol + eol
'-- 40-bit key
key      = Chr$(&h01,&h23,&h45,&h67,&h12)
plain    = Chr$(&h01,&h23,&h45,&h67,&h89,&hAB,&hCD,&hEF)
cipher   = zbs(Len(plain))
shouldbe = Chr$(&h7A,&hC8,&h16,&hD1,&h6E,&h9B,&h30,&h2E)
t = t + "Key length (bits):" + Str$((Len(key$)*8)) + eol
t = t + "plain:    " + Hex2Show$(plain) + eol
GoSub DoEncrypt
t = t + "cipher:   " + Hex2Show$(cipher) + eol
t = t + "shouldbe: " + Hex2Show$(shouldbe) + eol
key = Chr$(&h01,&h23,&h45,&h67,&h12)
GoSub DoDecrypt
t = t + "plain:    " + Hex2Show$(plain) + eol + eol
mbox(t)
ExitCC
Exit Function
'============
DoEncrypt:
ctx.UserKey       = StrPtr(key$)
ctx.UserKeyLength = Len(key$)
ctx.InBlock       = StrPtr(plain$)
ctx.OutBlock      = StrPtr(cipher$)
If Set_Key&(ctx) Then
   For i = 1 To (Len(plain$)\%BLOCKSIZE)  '<-- Required: (len(plain$) mod 8) = 0
      EncryptBlock ctx
      ctx.InBlock  = ctx.InBlock  + %BLOCKSIZE
      ctx.OutBlock = ctx.OutBlock + %BLOCKSIZE
   Next i&
Else
   mbox("Key setup error")
End If
'-- Burn the subkey
Poke$ Ctx.K, zbs(SizeOf(Ctx.K_Buffer))
Return
'============
DoDecrypt:
ctx.UserKey       = StrPtr(key$)
ctx.UserKeyLength = Len(key$)
ctx.InBlock       = StrPtr(cipher$)
ctx.OutBlock      = StrPtr(plain$)
If Set_Key&(ctx) Then
   For i = 1 To (Len(plain$)\%BLOCKSIZE)  '<-- Required: (len(plain$) mod 8) = 0
      DecryptBlock ctx
      ctx.InBlock  = ctx.InBlock  + %BLOCKSIZE
      ctx.OutBlock = ctx.OutBlock + %BLOCKSIZE
   Next i
Else
   mbox("Setup error")
End If
'-- Burn the subkey
Poke$ Ctx.K, zbs(SizeOf(Ctx.K_Buffer))
Return
End Function
'====================

Function Hex2Show$(Buffer$)
Local t$, i&, b As Byte Ptr
b = StrPtr(Buffer$)
For i = 0 To Len(Buffer$)-1
   t = t + Hex$(@b[i],2) + " "
Next i
Function = t
End Function
'-- end TESTBED.BAS



%NOGDI     = 1 ' no GDI (Graphics Device Interface) functions
%NOMMIDS   = 1 ' no Multimedia ID definitions
%NONEWWAVE = 1 ' no new waveform types except WAVEFORMATEX are defined
%NONEWRIFF = 1 ' no new RIFF formats are defined
%NOJPEGDIB = 1 ' no JPEG DIB definitions
%NONEWIC   = 1 ' no new Image Compressor types are defined
%NOBITMAP  = 1 ' no extended bitmap info header definition
#Include "WIN32API.INC"
'--------------------
'-- Utility macros
'--------------------
#If %Def(%PB_Win32)
   Macro eol=$Cr
   Macro mbox(t)=MsgBox t
#ElseIf %Def(%PB_CC32)
   Macro eol=$CrLf
   Macro mbox(t)=stdout t
#EndIf
'--------------------

Macro EnterCC
#If %Def(%PB_CC32)
   Local launched&
   If (cursory = 1) And (cursorx = 1) Then launched = -1
#EndIf
End Macro
'--------------------

Macro ExitCC
#If %Def(%PB_CC32)
   If launched Then
      Input Flush
      stdout "Press any key to end"
      WaitKey$
   End If
#EndIf
End Macro
''--------------------
